public class BinarySearchTree<E extends Comparable<E>> {
	private class Node {
		E data;
		Node leftChild = null, rightChild = null;
		// Konstukor, tar datan som ska vara i noden som argument.
		Node(E x) {
			data = x;
		}
	}
  // Trädet har initialt ett nod (kallas rot) som är null.
	Node root = null;

	public void add(E x) {
		Node cur = root;
		Node parent = null;
		int c = 0;
		while (cur != null) {
			c = x.compareTo(cur.data);
			if (c == 0) {  // found, already in set
				return;
			} else if (c < 0) {  // x less than current node's value, should be/is in left subtree
				parent = cur;
				cur = cur.leftChild;
			} else {  // c > 0, x greater than current node's value, should be/is in right subtree
				parent = cur;
				cur = cur.rightChild;
			}
		}
		// x is not in set and should be added in node which is child of parent
		// if last comparison (c) gave -1 than it should be as left child otherwise (c = 1) right child
		// if tree is empty then c = 0
		Node n = new Node(x);
		if (c < 0) {
			parent.leftChild = n;
		} else if (c > 0) {
			parent.rightChild =n;
		} else {  // c == 0
			root = n;
		}
	}
	public boolean contains(E x) {
		Node cur = root;
		while (cur != null) {
			int c = x.compareTo(cur.data);
			if (c == 0) {  // found
				return true;
			} else if (c < 0) {  // x less than current node's value, is in left subtree if anywhere
				cur = cur.leftChild;
			} else {  // c > 0, x greater than current node's value, is in right subtree if anywhere
			  cur = cur.rightChild;
			}
		}
		return false;
	}

	public boolean containsRec(E x) {
		return containsRec(root, x);
	}
	private boolean containsRec(Node cur, E x) {
		if (cur == null) {
			return false;
		} else {
			int c = x.compareTo(cur.data);
			if (c == 0) {  // found
				return true;
			} else if (c < 0) {  // x less than current node's value, is in left subtree if anywhere
				return containsRec(cur.leftChild, x);
			} else {  // c > 0, x greater than current node's value, is in right subtree if anywhere
				return containsRec(cur.rightChild, x);
			}
		}
	}

	public void printSorted() {
		printSorted(root);
		System.out.println();
	}
	private void printSorted(Node n) {
		if (n != null){
			printSorted(n.leftChild);
			System.out.println(n.data.toString() + " ");
			printSorted(n.rightChild);
		}
	}

	public static void main(String[] args) {
		Integer[] arr = {18, 8, 5, 2, 7, 13, 10, 17, 35, 25, 21, 19, 32, 39, 36, 42};

		BinarySearchTree<Integer> bst = new BinarySearchTree<>();

		for (Integer x : arr) {
			bst.add(x);
		}

		bst.printSorted();
	}
}
